/*
 * PjViewer.java
 *
 * Created on December 9, 2004, 6:49 PM
 */

package Grapher2D.Viewer;

import java.awt.Color;
import java.util.Hashtable;
import java.util.Enumeration;
import jv.function.PuFunction;
import jv.object.PsDebug;
import jv.project.PjProject;
import jv.vecmath.PdVector;
import jv.project.PvDisplayIf;
import jv.project.PvCameraIf;
import jv.number.PuDouble;
import Grapher2D.geom.*;
import jv.vecmath.PdMatrix;
import jv.geom.*;
import Grapher2D.util.PuDomain;
import jv.thirdParty.ruler.*;


/**
 * Describes a parametrized surface over a two dimensional domain.
 * Coordinate functions may be given as strings and
 * interactively edited. Parameter domain allows adjusting using sliders.
 * 
 * Based on work by Konrad Polthier
 * 
 * @author		David Lippman
 *
 */
public class PjViewer extends PjProject {
        public final static int TYPE_FUNC = 1; /**f(x,y) type*/
        public final static int TYPE_PIECE = 2; /**Parametric Surface type*/   
        public final static int TYPE_PCURVE = 3; /**Parametric Curve type*/
        public final static int TYPE_IMPSURF= 4; /**Implicit Curve type*/
        public final static int TYPE_IMPPOLAR = 7; /**Implicit Polar type*/
        public final static int TYPE_VF= 5; /**Vector Field on grid type*/
        public final static int TYPE_VFOS = 6; /**Vector Field on surface type*/
    
        protected       Hashtable               m_graphs;
        protected       PuDomain                m_clipbounds;
        protected       PuDouble                m_bClip;
	protected	PgElementSet			m_blackboard;
        protected       PgAxes                  axes;
        
        protected       int                     m_gDef = 1;
        
	public PjViewer() {
		super("2D Graph");

                m_graphs = new Hashtable();
                m_blackboard = new PgElementSet(2);                        
		if (getClass() == PjViewer.class)
			init();
	}
	public void init() {
                
//		m_blackboard.computePlane(6, 6, -5., -5., 5., 5.);
		m_blackboard.setName("X-Y Plane");
		m_blackboard.showElements(false);
		m_blackboard.setGlobalEdgeColor(new Color(161, 161, 161));
		m_blackboard.setGlobalElementColor(Color.white);
                
                m_clipbounds = new PuDomain();
                m_clipbounds.setVars(new String[] {"x","y"});
                m_clipbounds.setDefBnds(new double[] {-1000.0,1000.0,1.,5.});
                m_clipbounds.setDefVals(new double[] {-5.,5.});
                m_clipbounds.setNumDiscr(0);
                m_clipbounds.init();
                m_clipbounds.setParent(this);
                
		super.init();
 
	}
	public void start() {
            
                PvDisplayIf disp = getDisplay();
                if (disp != null) {
                        disp.selectCamera(PvCameraIf.CAMERA_ORTHO_XY);
                        disp.setBackgroundColor(Color.white);
                       updateClip();
                       setClip(true);      

                        disp.fit();
		}
                
                addGeometry(m_blackboard);
                
                             
		super.start();
	}

	/**
	 * Update the class whenever a child has changed.
	 * Method is usually invoked from the children.
	 */
	public boolean update(Object event) {
                if (event == this){
                    return super.update(this);
                } else if (event == m_clipbounds) {
                    updateClip();
                    setChildDomains();
                    return true;
                }
                
		return super.update(event);
	}    
        
        public boolean addGraph(String graphname, int type) {

        switch (type) {
            case TYPE_FUNC:
                PgGraph newfunc = new PgGraph(graphname);
                m_graphs.put(graphname,newfunc);
                addGeometry(newfunc);
                selectGeometry(newfunc);
                m_gDef++;
                return true;
             case TYPE_PCURVE:
                PgPCurve newpcurve = new PgPCurve(graphname);
                m_graphs.put(graphname,newpcurve);
                addGeometry(newpcurve);
                selectGeometry(newpcurve);
                m_gDef++;
                return true;
            case TYPE_IMPSURF:
                PgImpSurf newimpsurf = new PgImpSurf(graphname,TYPE_IMPSURF);
                m_graphs.put(graphname,newimpsurf);
                addGeometry(newimpsurf);
                selectGeometry(newimpsurf);
                m_gDef++;
                return true;
            case TYPE_IMPPOLAR:
                PgImpSurf newimppolar = new PgImpSurf(graphname,TYPE_IMPPOLAR);
                m_graphs.put(graphname,newimppolar);
                addGeometry(newimppolar);
                selectGeometry(newimppolar);
                m_gDef++;
                return true;
            case TYPE_PIECE:
                PgPiecewise newpw = new PgPiecewise(graphname);
                m_graphs.put(graphname,newpw);
                addGeometry(newpw);
                selectGeometry(newpw);
                m_gDef++;
                return true;
            case TYPE_VF:
                PgVecFld newvf = new PgVecFld(graphname);
                m_graphs.put(graphname, newvf);
                addGeometry(newvf);
                selectGeometry(newvf);
                m_gDef++;
                return true;
            case TYPE_VFOS:
                PgVecFldOn newvfon = new PgVecFldOn("VF on " + graphname);
                PgPointSet base = (PgPointSet) m_graphs.get(graphname);
                newvfon.setGeometry(base);
                base.addVectorField(newvfon);
                newvfon.update(newvfon);
                m_graphs.put("VF on "+graphname,newvfon);
                return true;
        }
                
        return false;
                
            
        }
/*        
        public boolean addVf(String vfbasename) {

                String graphname = "VF on " + vfbasename;
                

                m_graphs.put(graphname, (PjMulti) m_graphs.get(vfbasename));
 		PjMulti vfgraph = (PjMulti)m_graphs.get(vfbasename);
                if (vfgraph.addVf()) {
                     return true;
                } else {return false;}
        }
 */       
        public boolean removeGraph(String gname) {
                boolean hasvf = false;
                
                
                if (gname.startsWith("VF on ")) {   
                    String basename = gname.substring(6);
                    PgPointSet newgraph = (PgPointSet) m_graphs.get(basename);
                    newgraph.removeAllVectorFields();
                    newgraph.update(newgraph);
                } else {
                    PgPointSet newgraph = (PgPointSet) m_graphs.get(gname);
                    removeGeometry(newgraph);
                }
                
                if (m_graphs.containsKey("VF on " + gname)) {
                    m_graphs.remove("VF on " + gname);
                    hasvf = true;
                }
                m_graphs.remove(gname);
                return hasvf;
        }
        
        public void setScl() {
                PvDisplayIf disp = getDisplay();
                disp.setEnabledBoxRatio(true);
         
                
                double xsize = m_clipbounds.getMax(0) - m_clipbounds.getMin(0);
                double ysize = m_clipbounds.getMax(1) - m_clipbounds.getMin(1);
                
                double scl = Math.min(xsize,ysize);
    //            double scl = ysize;
                
                PdVector br = new PdVector(scl/xsize,scl/ysize,1);
 //               br.normalize();
                 disp.setBoxRatio(br);
                
 //               disp.setBoxRatio(new PdVector(xscl.getValue(),yscl.getValue(),zscl.getValue()));
                update(this);
                 
                disp.update(disp);
                setCamCenter();
        //        fitDisplays();
                
        }
        
        public void unsetScl() {
            PvDisplayIf disp = getDisplay();
                disp.setEnabledBoxRatio(false);
                
                disp.update(disp);
        }
        
        public void setCamCenter() {
            
            PvDisplayIf disp = getDisplay();
               
                
            
            double dist;
            if (disp.isEnabledBoxRatio()) {
                
                PvCameraIf cam = disp.getCamera();
                
                double xsize = m_clipbounds.getMax(0) - m_clipbounds.getMin(0);
                double ysize = m_clipbounds.getMax(1) - m_clipbounds.getMin(1);
                PdVector interest = new PdVector(m_clipbounds.getMin(0)+.5*xsize,m_clipbounds.getMin(1)+.5*ysize,0);
                cam.setInterest(interest);
                
                double scl = Math.min(xsize,ysize);
                
                dist = 1.2*scl;
                cam.setDist(dist);
                disp.update(cam);
            } else {
                disp.fit();
            }
             
            
               
        }
        public void updateClip() {
            double[] eps;
            eps = new double[2];
            eps[0] = (m_clipbounds.getMax(0)-m_clipbounds.getMin(0))/20.;
            eps[1] = (m_clipbounds.getMax(1)-m_clipbounds.getMin(1))/20.;
            int[] hashings = new int[2];

            double[] majorhash = new double[2];
            for (int i =0; i<2; i++) {
                            if (eps[i] < .2) {
                                majorhash[i] = 2*eps[i];
                            } else if (eps[i] < 1) {
                                majorhash[i] = 1.0;
                            } else if (eps[i] < 2) {
                                majorhash[i] = 2.0;
                            } else {
                                majorhash[i] = 5*(Math.ceil(eps[i]/2.5));
                            }
            }
  
            double[] planebox = new double[4];
                       planebox[0] = majorhash[0]*(Math.floor(m_clipbounds.getMin(0)/majorhash[0]));
                       planebox[1] = majorhash[1]*(Math.floor(m_clipbounds.getMin(1)/majorhash[1]));
                       planebox[2] = majorhash[0]*(Math.ceil(m_clipbounds.getMax(0)/majorhash[0]));
                       planebox[3] = majorhash[1]*(Math.ceil(m_clipbounds.getMax(1)/majorhash[1]));
                       hashings[0] = (int) Math.floor((planebox[2]-planebox[0])/majorhash[0]) + 1;
                       hashings[1] = (int) Math.floor((planebox[3]-planebox[1])/majorhash[1]) + 1;
             
            PvDisplayIf disp = getDisplay();
            for (int i=0; i<2;i++) {
//                    disp.setClipBounds(i, new double[] {m_clipbounds.getMin(i)-eps[i], m_clipbounds.getMax(i)+eps[i]});
                      disp.setClipBounds(i, new double[] {planebox[i]-eps[i], planebox[i+2]+eps[i]});
            }
            axes = ((jv.viewer.PvDisplay)disp).getAxes();
			if (axes == null) {
                            axes = new PgAxes(2);
                            axes.setDisplay(disp);
                            axes.setMode(PgAxes.AXES_2DXY_CENTER);
                            
                            PdVector [] bndbox = PdVector.realloc(null, 2, 3);
//                            bndbox[0].set(m_clipbounds.getMin(0)-eps[0],m_clipbounds.getMin(1)-eps[1], 0.); 
//                            bndbox[1].set(m_clipbounds.getMax(0)+eps[0],m_clipbounds.getMax(1)+eps[1], 0.); 
                            bndbox[0].set(planebox[0]-eps[0],planebox[1]-eps[1],0.);
                            bndbox[1].set(planebox[2]+eps[0],planebox[3]+eps[1],0.);
                            axes.setBounds(bndbox);
                            axes.setEnabledAutoBounds(false);
                            axes.setEnabledAutoHashing(false);
                            axes.setHashMode(PgRuler.HASH_FIXED); 
                            axes.setDisplay(disp);
                            ((jv.viewer.PvDisplay)disp).setAxes(axes);
                            disp.showAxes(true);
                            
                        } else {
                            axes.setEnabledAutoBounds(true);
                            PdVector [] bndbox = PdVector.realloc(null, 2, 3);
                            bndbox[0].set(planebox[0]-eps[0],planebox[1]-eps[1],0.);
                            bndbox[1].set(planebox[2]+eps[0],planebox[3]+eps[1],0.);
                              axes.setBounds(bndbox);
                            axes.setEnabledAutoBounds(false);
                        }    
            
                          
                      
                       axes.getAxes()[0].setFixedHashing(majorhash[0],majorhash[0]>2? majorhash[0]/5 : majorhash[0]/2);
                       axes.getAxes()[1].setFixedHashing(majorhash[1],majorhash[1]>2? majorhash[1]/5 : majorhash[1]/2);
                       axes.update(axes);

          
                       m_blackboard.computePlane(hashings[0],hashings[1], planebox[0] ,planebox[1],planebox[2] ,planebox[3] );
                       
//                       m_blackboard.computePlane((int)hashings[0], (int) hashings[1], m_clipbounds.getMin(0),m_clipbounds.getMin(1), m_clipbounds.getMax(0), m_clipbounds.getMax(1));
                        m_blackboard.update(m_blackboard);
                        disp.fit();
                        
         
            if (disp.isEnabledBoxRatio()){
                setScl();
            }
            disp.update(disp);
        }
        
        public void setClip(boolean turnon) {
            if (turnon) {
                PvDisplayIf disp = getDisplay();
                disp.setEnabledClipBounds(true);
                disp.update(disp);
            } else {
                PvDisplayIf disp = getDisplay();
                disp.setEnabledClipBounds(false);
                disp.update(disp);
            }
        }
        
        public void setChildDomains() {
            for (Enumeration e = m_graphs.keys(); e.hasMoreElements();) {
                String key = (String) e.nextElement();
                if (key.startsWith("Func")) {
                    PgGraph graph = (PgGraph) m_graphs.get(key);
                    graph.setDomain(new double[] {m_clipbounds.getMin(0), m_clipbounds.getMax(0)});
                } else if (key.startsWith("Imp")) {
                    PgImpSurf graph = (PgImpSurf) m_graphs.get(key);
                    graph.setDomain(new double[] {m_clipbounds.getMin(0), m_clipbounds.getMax(0), m_clipbounds.getMin(1), m_clipbounds.getMax(1)});
                } else if (key.startsWith("Vec")) {
                    PgVecFld graph = (PgVecFld) m_graphs.get(key);
                    graph.setDomain(new double[] {m_clipbounds.getMin(0), m_clipbounds.getMax(0), m_clipbounds.getMin(1), m_clipbounds.getMax(1)});
         
                }
            }
        }
        
    
}

