/*
 * PuDomain.java
 *
 * Created on December 5, 2004, 6:55 PM
 */

package Grapher2D.util;

import jv.object.PsObject;
import jv.function.PuFunction;
import jv.number.PuDouble;
import jv.number.PdColor;
import jv.number.PuInteger;
import jv.object.PsDebug;
import jv.vecmath.PdVector;
import jv.vecmath.PiVector;
/**
 *
 * @author David
 */
public class PuDomain extends PsObject {
        protected       double[]                        m_defVals = {-5.0,5.0};
        protected       double[]                        m_defBnds = {-20.0,20.0,.1,1.};
        protected       int[]                           m_defDiscr = {20};
        protected       int                             m_type;
        protected       int                             numvariables;
        protected       boolean                         m_bOneDiscr = false;
        protected       int                             m_numDiscr = -1;
        protected       boolean                         m_bIsVF = false;
        protected       boolean                         m_bAllowSlices = false;
        protected       String[]                        m_varNames = {"x","y"};
        protected       PuDouble[][]                    m_var;
        protected       double[]                        m_varValBU;
        protected       PuInteger[]                     m_discr;
        protected       PuDouble                        m_veclength;
        protected       boolean                         m_bSclVecs = false;
        protected       boolean[]                       m_bShowSlice;

        
    /** Creates a new instance of PuDomain */
    public PuDomain() {
        super();
        if (getClass() == PuDomain.class) {
            
        }
    }
    
    public void init() {
        super.init();
       	numvariables = m_varNames.length;
        m_var = new PuDouble[numvariables][2];
        m_varValBU = new double[numvariables];
        
//        m_discr = new PuInteger[m_bOneDiscr? 1 : numvariables];
        if (m_numDiscr == -1) {
            m_discr = new PuInteger[numvariables];
        } else {
            m_discr = new PuInteger[m_numDiscr];
        }
        
        m_bShowSlice = new boolean[numvariables];
        
        for (int i=0; i<numvariables;i++) {
            m_var[i][0] = new PuDouble(m_varNames[i] + " min", this);
            m_var[i][0].setDefBounds(m_defBnds[0],m_defBnds[1],m_defBnds[2],m_defBnds[3]);
            if (m_defVals.length == 2) {
                m_var[i][0].setDefValue(m_defVals[0]);
            } else {
                m_var[i][0].setDefValue(m_defVals[2*i]);
            }
            m_var[i][0].init();
            m_var[i][1] = new PuDouble(m_varNames[i] + " max", this);
            m_var[i][1].setDefBounds(m_defBnds[0],m_defBnds[1],m_defBnds[2],m_defBnds[3]);
            if (m_defVals.length == 2) {
                m_var[i][1].setDefValue(m_defVals[1]);
            } else {
                m_var[i][1].setDefValue(m_defVals[2*i+1]);
            }
            m_var[i][1].init();
            if (m_numDiscr == -1) {
//            if (!m_bOneDiscr) {
                m_discr[i] = new PuInteger(m_varNames[i] + " discr", this);
                m_discr[i].setDefBounds(2,300,1,5);
                if (m_defDiscr.length == 1) {
                     m_discr[i].setDefValue(m_defDiscr[0]);
                } else {
                    m_discr[i].setDefValue(m_defDiscr[i]);
                }
                m_discr[i].init();
            }
            m_bShowSlice[i] = false;
        }
        if (m_numDiscr ==1) {
//        if (m_bOneDiscr) {
            m_discr[0] = new PuInteger("discr", this);
            m_discr[0].setDefBounds(2,300,1,5);
            m_discr[0].setDefValue(m_defDiscr[0]);
            m_discr[0].init();
           
        }
             
        if (m_bIsVF) {
            m_veclength  = new PuDouble("Vector Length",this);
            m_veclength.setDefBounds(0,4,0.1,0.5);
            m_veclength.setDefValue(1);
            m_veclength.init();
        }
        
    }
    
 
    public void setDefVals(double[] vals) {
        m_defVals = vals;
    }
    
    public void setDefBnds(double[] bnds) {
        m_defBnds = bnds;
    }
    
    public void setDefDiscr(int[] discr) {
        m_defDiscr = discr;
    }
    
    public void setVals(double[] vals) {
        for (int i=0; i<numvariables;i++) {
            m_var[i][0].setValue(vals[2*i]);
            m_var[i][1].setValue(vals[2*i+1]);
        }
    }
    
    public void setVars(String[] vars) {
        m_varNames = vars;
    }
    
    public void setAllowSlices(boolean allow) {
        m_bAllowSlices = allow;
    }
    
    public void setOneDiscr(boolean one) {
        m_bOneDiscr = one;
    }
    
    /** Set =0 for no Discr, 1 for 1 Discr, -1 for Discr for each variable */
    public void setNumDiscr(int num) {
        m_numDiscr = num;
    }
    
    public void setIsVF(boolean is) {
        m_bIsVF = is;
    }
    
    public boolean isSclVecs() {
        return m_bSclVecs;
    }
    
    public double getVeclength() {
        return m_veclength.getValue();
    }
    
    public int getDiscr(int i) {
        return m_discr[i].getValue();
    }
    
    public int[] getDiscr() {
        int[] discr;
        discr = new int[numvariables];
        for (int i=0; i<numvariables; i++) {
            discr[i] = m_discr[i].getValue();
        }
        return discr;
    }
    
    public double getMin(int i) {
        return m_var[i][0].getValue();
    }
    
    public double getMax(int i) {
        return m_var[i][1].getValue();
    }
    
    public boolean isSliced(int i) {
        return m_bShowSlice[i];
    }
    
    public void setEnabled(boolean set) {
        
       for (int i=0; i < numvariables; i++) {
           m_var[i][0].setEnabled(set);
           m_var[i][1].setEnabled(set);
       }
        
    }
    
    public boolean update(Object event) {
                if (event == this) {
                        super.update(this);
                        return true;
		} else if (event == m_discr) {
                        
			update(this);
			return true;
		} 
                for (int i=0; i<numvariables;i++) {
                    if (event == m_var[i][0]) {
                        if (m_var[i][0].getValue() >= m_var[i][1].getValue()) {
                            m_var[i][0].setValue(m_var[i][1].getValue()-.1);
                        }
                    } else if (event == m_var[i][1]) {
                        if (m_var[i][0].getValue() >= m_var[i][1].getValue()) {
                            m_var[i][1].setValue(m_var[i][0].getValue()+.1);
                        }
                    }
                }
                
                super.update(this);
                
		return true;
	}
    
        
    
}
